// -*- Mode: vala; indent-tabs-mode: nil; tab-width: 4 -*-
/*-
 * Copyright (c) 2011-2015 Footnote Developers (http://launchpad.net/footnote)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Corentin Noël <corentin@elementaryos.org>
 *              Artem Anufrij <artem.anufrij@live.de>
 */

public class Footnote.DataBase.DatabaseManager : GLib.Object {

    private static DatabaseManager database;
    public static DatabaseManager get_default () {
        if (database == null)
            database = new DatabaseManager ();

        return database;
    }

    private SQLHeavy.Database db;
    GLib.File db_file;

    Footnote.Settings settings;

    private DatabaseManager () {

        settings = Footnote.Settings.get_default ();

        if (settings.database_location == "")
            settings.database_location = Environment.get_user_data_dir () + "/footnote";

        db_file = File.new_for_path (settings.database_location + "/notes.db");

        connect_database (!db_file.query_exists ());
    }

    /*
     * Notes functions.
     */

    private void connect_database (bool need_create) {

        try {
            db = new SQLHeavy.Database (db_file.get_path (),
                SQLHeavy.FileMode.READ | SQLHeavy.FileMode.WRITE | SQLHeavy.FileMode.CREATE);
            if (need_create) {
                db.execute ("CREATE TABLE notebooks (name TEXT, color TEXT, id INTEGER PRIMARY KEY AUTOINCREMENT);");
                db.execute ("CREATE VIRTUAL TABLE notes USING fts3(title TEXT, content BLOB, notebook INTEGER, bookmarked BOOL, date INT64);");
            }
        } catch (SQLHeavy.Error e) {
            critical ("Could not create db: %s", e.message);
        }
    }

    public void add_note (Footnote.Objects.Note note) {
        try {
            var q = db.prepare("INSERT INTO 'notes' ( title, content, notebook, date, bookmarked) VALUES  (:title, :content, :notebook, :date, :bookmarked);");
            q.set_string (":title", _("Untitled"));
            q.set_string (":content", "");
            q.set_int (":notebook", note.notebook.id);
            q.set_int64 (":date", note.datetime.to_unix ());
            q.set_int (":bookmarked", (int)note.bookmarked);
            q.execute ();
            note.id = (int)db.last_insert_id;
        } catch (SQLHeavy.Error e) {
            critical (e.message);
        }
    }

    public void update_note (Footnote.Objects.Note note) {
        try {
            DateTime datetime = new DateTime.now_local ();
            Gtk.TextIter s, e;
            note.content.get_bounds (out s, out e);
            uint8 [] data = note.content.serialize (note.content, note.format, s, e);
            var q = db.prepare ("UPDATE 'notes' SET title=:title, content=:content, bookmarked=:bookmarked, date=:date WHERE rowid=:id");
            q.set_string (":title", note.title.text);
            q.set_blob (":content", data);
            q.set_int (":bookmarked", (int)note.bookmarked);
            q.set_int64 (":date", datetime.to_unix ());
            q.set_int (":id", note.id);
            q.execute ();
            note.datetime = datetime;
            note.saved ();
        } catch (SQLHeavy.Error e) {
            critical (e.message);
        }
    }

    public void delete_note (Footnote.Objects.Note note) {
        try {
            var q = db.prepare ("DELETE FROM 'notes' WHERE rowid = :id;");
            q.set_int (":id", note.id);
            q.execute ();
        } catch (SQLHeavy.Error e) {
            critical (e.message);
        }
    }

    public void restore_note (Footnote.Objects.Note note) {

        try {
            var q = db.prepare ("SELECT title, content, rowid, date, bookmarked FROM 'notes' WHERE rowid=:id");
            q.set_int (":id", note.id);

            var res = q.execute ();

            DateTime datetime = datetime_converter (res, 3);

            // TITLE
            note.title.text = res.fetch_string (0);

            // CONTENT
            if (res.fetch_blob (1) != null) {
                Gtk.TextIter start;
                Gtk.TextIter end;

                // Clear currently content
                note.content.get_bounds (out start, out end);
                note.content.delete_range (start, end);

                note.content.get_start_iter (out start);
                try {
                    note.content.deserialize (note.content, note.deformat, start, res.fetch_blob (1));
                } catch (Error e) {
                    critical (e.message);
                }
            }

            // DATETIME
            note.datetime = datetime;

            // BOOKMARKED
            note.bookmarked = (bool)res.fetch_int (4);
            note.restored ();
        } catch (SQLHeavy.Error e) {
            critical (e.message);
        }
    }

    public Gee.ArrayList<Footnote.Objects.Note> get_notes_from_notebook (Footnote.Objects.Notebook nb) {
        var list = new Gee.ArrayList<Footnote.Objects.Note> ();
        try {
            var q = db.prepare ("SELECT title, content, rowid, date, bookmarked FROM 'notes' WHERE notebook = :id ORDER BY rowid ASC;");
            q.set_int (":id", nb.id);
            for (var res = q.execute (); !res.finished; res.next ()) {
                DateTime datetime = datetime_converter (res, 3);

                var note = new Footnote.Objects.Note.from_existing (nb, res.fetch_string (0), res.fetch_blob (1),
                    res.fetch_int (2), datetime, (bool)res.fetch_int (4));
                list.add (note);
            }
        } catch (SQLHeavy.Error e) {
            warning ("Could not query the notes: %s", e.message);
        }

        return list;
    }

    public void move_note_to_notebook (Footnote.Objects.Note note, Footnote.Objects.Notebook nb) {
        try {
            var q = db.prepare ("UPDATE 'notes' SET notebook=:cat WHERE rowid=:id");
            q.set_int (":cat", nb.id);
            q.set_int(":id", note.id);
            q.execute();
        } catch (SQLHeavy.Error e) {
            critical (e.message);
        }
    }

    /*
     * Notebook functions.
     */

    public Gee.ArrayList<Footnote.Objects.Notebook> get_notebooks () {
        var list = new Gee.ArrayList<Footnote.Objects.Notebook> ();
        try {
            var app = (FootnoteApp)GLib.Application.get_default ();
            for (var res=db.execute ("SELECT name,color,id FROM 'notebooks';"); !res.finished ; res.next ()) {
                var rgba = Gdk.RGBA ();
                // Keep backward compatibility, but now it's stored as a string.
                if (res.field_type (1) == typeof(int)) {
                    var color = (uint32)res.fetch_int (1);
                    uint32 r = color>>24;
                    uint32 g = color>>16;
                    uint32 bl = color>>8;
                    rgba.red = (double)((uint8)r)/255.0;
                    rgba.green = (double)((uint8)g)/255.0;
                    rgba.blue = (double)((uint8)bl)/255.0;
                } else {
                    var color_string = res.fetch_string (1);
                    rgba.parse (color_string);
                }
                var notebook = new Footnote.Objects.Notebook.from_existing  (app, res.fetch_string (0), rgba, res.fetch_int(2));
                list.add (notebook);
            }
        } catch (SQLHeavy.Error e) {
            warning ("Could not query the notebooks: %s", e.message);
            show_critical_error ();
        }

        return list;
    }

    public void add_notebook (Footnote.Objects.Notebook nb) {
        try {
            var q = db.prepare ("INSERT INTO 'notebooks' (name, color) VALUES (:name, :color);");
            q.set_string (":name", nb.title);
            q.set_string (":color", nb.color.to_string ());
            q.execute ();
            nb.id = (int)db.last_insert_id;
        } catch (SQLHeavy.Error e) {
            critical (e.message);
        }
    }

    public void update_notebook (Footnote.Objects.Notebook nb) {
        try {
            var q = db.prepare ("UPDATE 'notebooks' SET name=:name, color=:color WHERE id=:id");
            q.set_string (":name", nb.title);
            q.set_string (":color", nb.color.to_string ());
            q.set_int(":id", nb.id);
            q.execute ();
            nb.modified ();
        } catch (SQLHeavy.Error e) {
            critical (e.message);
        }
    }

    public void delete_notebook (Footnote.Objects.Notebook nb) {
        try {
            var q = db.prepare ("DELETE FROM 'notebooks' WHERE id = :id;");
            q.set_int (":id", nb.id);
            q.execute ();
            var r = db.prepare ("DELETE FROM 'notes' WHERE notebook = :id;");
            r.set_int (":id", nb.id);
            r.execute ();
        } catch (SQLHeavy.Error e) {
            critical (e.message);
        }
    }

    private void show_critical_error () {
        var dialog = new Gtk.Dialog.with_buttons ("Problem", null, Gtk.DialogFlags.MODAL, _("Close"), Gtk.ResponseType.OK);
        var box = new Gtk.Box (Gtk.Orientation.HORIZONTAL, 0);
        ((Gtk.Container)dialog.get_content_area ()).add (box);

        var label = new Gtk.Label ("""Footnote could not query the notebooks.
        If this happens after an update you either want to delete the old
        database at $home/.local/share/footnote/notes.db or, if you already
        have (very) important data in the database,
        <a href='https://bugs.launchpad.net/footnote/+filebug'>report a bug</a>
        asking for a converting tool, we will see what we can do.""");
        label.wrap_mode = Pango.WrapMode.WORD;
        label.wrap = true;
        label.use_markup = true;
        label.margin_left = 12;

        box.margin = 12;
        box.pack_start (new Gtk.Image.from_icon_name ("dialog-error", Gtk.IconSize.DIALOG), false);
        box.pack_start (label);

        dialog.show_all ();
        dialog.run ();
        dialog.destroy ();
    }

    public DateTime datetime_converter (SQLHeavy.QueryResult res, int column_index) {
        DateTime return_value = new DateTime.now_local ();

        try {
            if (res.field_type (column_index) == typeof(string)) {
                var date_string = res.fetch_string (column_index);
                var parts = date_string.split (" ", 3);
                var hour_parts = parts[0].split (":", 2);
                var day_parts = parts[2].split (".", 3);
                var year = int.parse (day_parts[2]);
                var month = int.parse (day_parts[1]);
                var day = int.parse (day_parts[0]);
                var hours = int.parse (hour_parts[0]);
                var minutes = int.parse (hour_parts[1]);

                return_value = new DateTime.local (year, month, day, hours, minutes, 0);
            } else
                return_value = new DateTime.from_unix_local (res.fetch_int64 (column_index));
        } catch (SQLHeavy.Error e) {
            critical (e.message);
        }

        return return_value;
    }

    public void move_database (string new_location) {
        try {
            db_file.move(File.new_for_path (new_location + "/notes.db"), GLib.FileCopyFlags.OVERWRITE);

            settings.database_location = new_location;

            db_file = File.new_for_path (settings.database_location + "/notes.db");

            connect_database (false);
        } catch (Error e) {
            error (e.message);
        }
    }
}
