//
//  Copyright (C) 2011-2012 Tom Beckmann
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//  Authored by:    Artem Anufrij <artem.anufrij@live.de>

using Footnote.Dialogs;
using Footnote.Widgets;
using Footnote.Objects;

class LLabel : Gtk.Label {
    public LLabel (string label) {
        this.set_halign (Gtk.Align.START);
        this.label = label;
    }
    public LLabel.indent (string label) {
        this (label);
        this.margin_left = 10;
    }
    public LLabel.markup (string label) {
        this (label);
        this.use_markup = true;
    }
    public LLabel.r (string label) {
        this.set_halign (Gtk.Align.END);
        this.label = label;
    }
    public LLabel.right_with_markup (string label) {
        this.set_halign (Gtk.Align.END);
        this.use_markup = true;
        this.label = label;
    }
}


namespace Footnote {
    /**
     * the app and everything else
     **/
    public class FootnoteApp : Granite.Application {

        public Settings                     settings;

        public List<Notebook>               notebooks;

        public Gtk.Window                   mainwindow;
        public Widgets.SideBar              sidebar;
        public Granite.Widgets.Welcome      welcome;
        public Gtk.Paned                    hmain;
        private Gtk.Stack stack;

        public Gtk.SearchEntry              search;
        public Gtk.ToolButton               add_button;
        public Gtk.ToolButton               remove_button;
        public Gtk.ToolButton               edit_button;

        public Gtk.TreeIter                 notebooks_iter;
        public Gtk.TreeIter                 bookmarks_iter;

        private Widgets.MonthBar            topbar;
        private Notebook selected_notebook {
            get {
                var item = sidebar.selected;
                if (item is Widgets.NotebookItem) {
                    return ((Widgets.NotebookItem)item).notebook;
                } else { //bookmarks
                    return ((Widgets.BookmarkItem)item).note.notebook;
                }
            }
        }

        public bool                         showing_search;
        public bool                         fullscreened;
        public Gtk.InfoBar                  undo_bar;
        private Gtk.ToolButton              add_new_button;

        public int                          year;
        public int                          month;

        construct {
            program_name = "Footnote";
            exec_name = "footnote";

            build_data_dir = Constants.DATADIR;
            build_pkg_data_dir = Constants.PKGDATADIR;
            build_release_name = Constants.RELEASE_NAME;
            build_version = Constants.VERSION;
            build_version_info = Constants.VERSION_INFO;

            app_years = "2011-2015";
            app_icon = "footnote";
            app_launcher = "footnote.desktop";
            application_id = "net.launchpad.Footnote";

            main_url = "https://code.launchpad.net/footnote";
            bug_url = "https://bugs.launchpad.net/footnote";
            help_url = "https://code.launchpad.net/footnote";
            translate_url = "https://translations.launchpad.net/footnote";

            about_authors = {"Tom Beckmann <tombeckmann@online.de>"
                            ,"Artem Anufrij <artem.anufrij@live.de>"};
            about_documenters = {"Tom Beckmann <tombeckmann@online.de>"};
            about_artists = {"Harvey Cabaguio"};
            about_comments = "Development release, not all features implemented";
            about_translators = "";
            about_license_type = Gtk.License.GPL_3_0;
        }

        public static FootnoteApp _instance = null;

        public static FootnoteApp instance {
            get {
                if (_instance == null)
                    _instance = new FootnoteApp ();
                return _instance;
            }
        }

        protected override void activate () {

            if (mainwindow != null) {
                mainwindow.present (); // present window if app is already running
                return;
            }

            debug ("Build new Window");

            month = -1;
            year  = -1;

            notebooks = new List<Notebook> ();

            build ();

            var notebooks_list = Footnote.DataBase.DatabaseManager.get_default ().get_notebooks ();
            foreach (var notebook in notebooks_list) {
                add_notebook (notebook);
            }

            if (this.notebooks.length () == 0) {
                var notebook = new Footnote.Objects.Notebook (this, _("Unsorted Notes"));
                notebook.color = {255, 0, 0};
                add_notebook (notebook);
                sidebar.selected = notebook.sidebar_item;
                add_new_note ();
            } else {
                selected_notebook.view.show_all ();
                selected_notebook.view.update_view ();
            }

            generate_year_combobox ();

            int width = settings.window_width, height = settings.window_height;
            Settings.WindowState window_state = settings.window_state;
            mainwindow.configure_event.connect ((event) => {
                if (mainwindow.get_window ().get_state () == Gdk.WindowState.MAXIMIZED) {
                    window_state = Settings.WindowState.MAXIMIZED;
                } else {
                    mainwindow.get_size (out width, out height);
                    window_state = Settings.WindowState.NORMAL;
                }

                return false;
            });

            mainwindow.key_press_event.connect ((e) => {
                if (e.keyval == Gdk.Key.n && Gdk.ModifierType.CONTROL_MASK in e.state) {
                    add_new_note ();
                    return true;
                } else if (e.keyval == Gdk.Key.F2) {
                    focus_selected_note ();
                    return true;
                }
                return false;
            });

            mainwindow.destroy.connect (() => {
                settings.window_width = width;
                settings.window_height = height;
                settings.window_state = window_state;

                settings.sidebar_width = hmain.position;
                if (selected_notebook != null) {
                    settings.last_notebook = selected_notebook.id;
                    selected_notebook.view.note_editor.save ();
                }

                Gtk.main_quit ();
            });

            Gtk.main ();
        }

        private void build () {
            mainwindow = new Gtk.Window ();
            settings = Settings.get_default ();
            settings.changed.connect (settings_changed);
            undo_bar = new Gtk.InfoBar.with_buttons (_("Undo"), Gtk.ResponseType.CANCEL);

            /*
             * Header Bar
             */
            add_new_button = new Gtk.ToolButton (null, null);
            add_new_button.icon_name = "document-new";
            add_new_button.name = _("New Note");
            add_new_button.tooltip_text = _("Create a new note");
            add_new_button.clicked.connect (() => {
                add_new_note ();
            });

            var export_button = new Gtk.ToolButton (null, null);
            export_button.icon_name = "document-export";
            export_button.name = _("Export notebook");
            export_button.tooltip_text = _("Export current notebook…");

            var settings_button = new Gtk.ToolButton (null, null);
            settings_button.icon_name = "document-properties";
            settings_button.name = _("Settings");
            settings_button.tooltip_text = _("Settings");
            settings_button.clicked.connect (() => {
                SettingsDialog dialog = new SettingsDialog (mainwindow);
                dialog.show_all ();
                dialog.run ();
                dialog = null;
            });
            search = new Gtk.SearchEntry ();
            search.placeholder_text = _("Search Notes");

            var tb = new Gtk.HeaderBar ();
            tb.show_close_button = true;
            tb.pack_start (add_new_button);
            tb.pack_end (settings_button);
            tb.pack_end (export_button);
            tb.pack_end (search);

            mainwindow.set_titlebar (tb);

            /*
             * Left view
             */
            sidebar = new Widgets.SideBar ();

            add_button = new Gtk.ToolButton (null, null);
            add_button.icon_name = "list-add-symbolic";

            remove_button = new Gtk.ToolButton (null, null);
            remove_button.icon_name = "list-remove-symbolic";

            edit_button = new Gtk.ToolButton (null, null);
            edit_button.icon_name = "edit-symbolic";

            var buttons_tb = new Gtk.Toolbar ();
            buttons_tb.icon_size = Gtk.IconSize.MENU;
            buttons_tb.get_style_context ().add_class (Gtk.STYLE_CLASS_INLINE_TOOLBAR);
            buttons_tb.insert (add_button,    -1);
            buttons_tb.insert (remove_button, -1);
            buttons_tb.insert (edit_button,    -1);

            var left_box = new Gtk.Box (Gtk.Orientation.VERTICAL, 0);
            left_box.pack_start (sidebar, true, true, 0);
            left_box.pack_start (buttons_tb, false, false, 0);

            /*
             * Right view
             */
            topbar = new Widgets.MonthBar ();
            topbar.month_changed.connect ((m) => {set_selected_month (m);});
            topbar.year_changed.connect ((y) => {update_monthbar_sensitivity (); set_selected_year (y);});

            stack = new Gtk.Stack ();
            var right_grid = new Gtk.Grid ();
            right_grid.orientation = Gtk.Orientation.VERTICAL;
            right_grid.add (topbar);
            right_grid.add (stack);

            hmain = new Gtk.Paned (Gtk.Orientation.HORIZONTAL);
            hmain.pack1 (left_box, false, true);
            hmain.pack2 (right_grid, true, true);
            hmain.position = settings.sidebar_width;

            mainwindow.title = program_name;
            mainwindow.icon_name = app_icon;
            mainwindow.height_request = 350;
            mainwindow.width_request = 400;
            mainwindow.window_position = Gtk.WindowPosition.CENTER;
            mainwindow.set_default_size (settings.window_width, settings.window_height);
            mainwindow.add (hmain);
            mainwindow.show_all ();

            topbar.visible = settings.show_month_bar;

            // Maximize window if necessary
            switch (settings.window_state) {
                case Settings.WindowState.MAXIMIZED:
                    mainwindow.maximize ();
                    break;
                default:
                    break;
            }

            /**
             * events
             **/
            /*search on every keypress*/
            search.search_changed.connect (() => {
                on_search_changed ();
            });
            /*search icon pressed, start searching*/
            search.icon_press.connect ((entry, event) => {
                if (entry == Gtk.EntryIconPosition.SECONDARY)
                   on_search_changed ();
            });
            /*global notebook export*/
            export_button.clicked.connect (()=>{
                new ExportDialog.export_notes (selected_notebook.get_notes (), selected_notebook.title, mainwindow);
            });
            /*notebook selection changed*/
            sidebar.item_selected.connect ((item) => {
                if (item is Widgets.NotebookItem) { //notebooks
                    edit_button.sensitive = true;
                    add_button.sensitive = true;
                    var selected_item = ((Widgets.NotebookItem)item).notebook;
                    switch_to_notebook (selected_item);
                    if (settings.show_month_bar)
                        topbar.show ();
                } else if (item is Widgets.BookmarkItem) { //bookmarks
                    edit_button.sensitive = false;
                    add_button.sensitive = false;
                    var note = ((Widgets.BookmarkItem)item).note;
                    stack.set_visible_child (note.notebook.view);
                    note.notebook.view.open_note (note, true);
                    topbar.hide ();
                }
            });
            /*add a new notebook*/
            this.add_button.clicked.connect (() => {
                create_add_popover ();
            });

            /*edit current category*/
            this.edit_button.clicked.connect (() => {
                create_modify_popover (selected_notebook);
            });

            /*remove current notebook*/
            this.remove_button.clicked.connect (() => {
                var item = sidebar.selected;
                if (item is Widgets.NotebookItem) { //notebooks
                    var selected_item = ((Widgets.NotebookItem)item).notebook;
                    create_remove_popover (selected_item);
                } else if (item is Widgets.BookmarkItem) { //bookmarks
                    var note = ((Widgets.BookmarkItem)item).note;
                    note.bookmarked = false;
                    note.bookmark (note.bookmarked);
                }
            });
            /*DnD*/
            /*Gtk.TargetEntry e = {"footnote_note", 0, 0};
            Gtk.drag_dest_set (this.sidebar, Gtk.DestDefaults.ALL, {e}, Gdk.DragAction.MOVE);
            this.sidebar.drag_data_received.connect ( (widget, ctx, x, y, sel, info, timestamp) => {
                Gtk.TreePath path;
                Gtk.TreeIter iter;
                this.sidebar.get_dest_row_at_pos (x, y, out path, null);
                this.sidebar.model.get_iter (out iter, path);

                selected_notebook.remove_note (selected.note);

                var dest_cat = (Footnote.Objects.Notebook)this.sidebar.getObject (iter);
                dest_cat.add_note (selected.note);
                selected.note.notebook = dest_cat;

                selected_notebook.view.update_view ();
                DataBase.get_default ().move_note_to_notebook (selected.note, selected.note.notebook);

                Gtk.drag_finish (ctx, true, false, timestamp);
            });*/
        }

        private void settings_changed () {
            topbar.visible = settings.show_month_bar;
        }

        private void set_selected_month (int month) {
            foreach (var notebook in notebooks) {
                notebook.view.change_month (month);
            }
        }

        private void set_selected_year (int year) {
            foreach (var notebook in notebooks) {
                notebook.view.change_year (year);
            }
        }

        private void switch_to_notebook (Notebook notebook) {
            stack.set_visible_child (notebook.view);
            sidebar.selected = notebook.sidebar_item;
            selected_notebook.view.update_view ();
            generate_year_combobox ();
        }

        private void on_search_changed () {
            foreach (Notebook c in notebooks) {
                c.view.filter = search.text;
            }
            selected_notebook.view.update_view ();
        }

        private void add_notebook (Notebook notebook) {
            notebooks.append (notebook);
            stack.add (notebook.view);
            sidebar.add_notebook (notebook);
            notebook.note_added.connect ((note) => {
                generate_year_combobox ();
            });

            notebook.note_trashed.connect ((note) => {
                generate_year_combobox ();
            });
        }

        private void generate_year_combobox () {
            var years_set = new Gee.HashSet<int> ();
            if (sidebar.selected is Widgets.NotebookItem) {
                var notebook = ((Widgets.NotebookItem)sidebar.selected).notebook;
                foreach (var note in notebook.get_notes ()) {
                    if (note.is_trashed == false)
                        years_set.add (note.year);
                }

                topbar.generate_year_combobox (years_set);
                update_monthbar_sensitivity ();
            }
        }

        private void update_monthbar_sensitivity () {
            var months_set = new Gee.HashSet<int> ();
            if (sidebar.selected is Widgets.NotebookItem) {
                var notebook = ((Widgets.NotebookItem)sidebar.selected).notebook;
                foreach (var note in notebook.get_notes ()) {
                    if ((topbar.year == -1 || topbar.year == note.year) && note.is_trashed == false) {
                        months_set.add (note.month-1);
                    }
                }

                topbar.update_sensitivity (months_set);
            }
        }

        private void create_add_popover () {
            var name_label = new Gtk.Label (_("Name:"));
            name_label.halign = Gtk.Align.END;
            var name = new Gtk.Entry ();
            name.set_placeholder_text (_("Notebook name"));
            var ok = new Gtk.Button.with_label (_("Add"));
            ok.sensitive = false;
            ok.get_style_context ().add_class (Gtk.STYLE_CLASS_SUGGESTED_ACTION);
            var color_label = new Gtk.Label (_("Color:"));
            color_label.halign = Gtk.Align.END;
            var color_button = new Gtk.ColorButton ();

            var grid = new Gtk.Grid ();
            grid.margin = 6;
            grid.row_spacing = 6;
            grid.column_spacing = 12;
            grid.attach (name_label, 0, 0, 1, 1);
            grid.attach (name, 1, 0, 1, 1);
            grid.attach (color_label, 0, 1, 1, 1);
            grid.attach (color_button, 1, 1, 1, 1);
            grid.attach (ok, 0, 2, 2, 1);

            var popover = new Gtk.Popover (add_button);
            popover.add (grid);

            name.key_release_event.connect ((event) => {
                if (name.text != ""){
                    ok.sensitive = true;
                    if (event.keyval == 65293){
                        ok.clicked ();
                    }
                } else {
                    ok.sensitive = false;
                }

                return false;
            });

            ok.clicked.connect (() => {
                var notebook = new Footnote.Objects.Notebook (this, name.text);
                notebook.color = color_button.rgba;
                add_notebook (notebook);

                remove_button.sensitive = true;
                add_new_button.sensitive = true;
                popover.hide ();

                sidebar.selected = notebook.sidebar_item;
                add_new_note ();
            });

            popover.hide.connect (() => {
                popover.destroy ();
            });

            popover.show_all ();
        }

        private void create_remove_popover (Notebook notebook) {
            var dlg = new Gtk.Popover (remove_button);
            var b   = new Gtk.Box (Gtk.Orientation.VERTICAL, 6);
            var yes = new Gtk.Button.with_label (_("Delete"));

            b.pack_start (new Gtk.Label (_("Delete topic with all its notes?")));
            b.pack_start (yes);
            b.margin = 6;
            dlg.add (b);

            yes.get_style_context ().add_class (Gtk.STYLE_CLASS_DESTRUCTIVE_ACTION);
            yes.clicked.connect (() => {
                Footnote.DataBase.DatabaseManager.get_default ().delete_notebook (notebook);
                notebooks.delete_link (notebooks.find (notebook));
                sidebar.remove_notebook (notebook);

                foreach (var n in notebook.get_notes ()) {
                    Footnote.DataBase.DatabaseManager.get_default ().delete_note (n);
                }

                dlg.hide ();
            });

            dlg.hide.connect (() => {
                dlg.destroy ();
            });

            dlg.show_all ();
        }

        private void create_modify_popover (Notebook notebook) {
            var name_label = new Gtk.Label (_("Name:"));
            name_label.halign = Gtk.Align.END;
            var name = new Gtk.Entry ();
            name.set_placeholder_text (_("Notebook name"));
            name.text = notebook.title;
            var edit = new Gtk.Button.with_label (_("Edit"));
            var color_label = new Gtk.Label (_("Color:"));
            color_label.halign = Gtk.Align.END;
            var color_button = new Gtk.ColorButton.with_rgba (notebook.color);

            var grid = new Gtk.Grid ();
            grid.margin = 6;
            grid.row_spacing = 6;
            grid.column_spacing = 12;
            grid.attach (name_label, 0, 0, 1, 1);
            grid.attach (name, 1, 0, 1, 1);
            grid.attach (color_label, 0, 1, 1, 1);
            grid.attach (color_button, 1, 1, 1, 1);
            grid.attach (edit, 0, 2, 2, 1);

            var popover = new Gtk.Popover (edit_button);
            popover.add (grid);

            name.key_release_event.connect ((event) => {
                if (name.text != "") {
                    edit.sensitive = true;
                    if (event.keyval == 65293)
                        edit.clicked ();
                } else {
                    edit.sensitive = false;
                }

                return false;
            });

            popover.hide.connect (() => {
                popover.destroy ();
            });

            edit.clicked.connect (() => {
                if (name.text != "") {
                    notebook.title = name.text;
                    notebook.color = color_button.rgba;
                    remove_button.sensitive = true;
                    Footnote.DataBase.DatabaseManager.get_default ().update_notebook (notebook);
                }

                popover.hide ();
            });

            popover.show_all ();
        }

        private void add_new_note () {
            var n = new Note (selected_notebook);
            sidebar.select_notebook (selected_notebook);
            selected_notebook.add_note (n);
            selected_notebook.view.select_note (n);
        }

        private void focus_selected_note () {
            selected_notebook.view.note_editor.set_focus ();
        }
    }
}

public static int main (string [] args) {
    Gtk.init (ref args);
    var app = Footnote.FootnoteApp.instance;
    return app.run (args);
}
