//
//  Copyright (C) 2011-2012 Tom Beckmann
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//  Authored by:    Artem Anufrij <artem.anufrij@live.de>


namespace Footnote.Objects {
    /**
     * class representing a note
     **/
    public class Note : GLib.Object {
        public signal void changed ();
        public signal void saved ();
        public signal void restored ();
        public signal void trashed ();
        public signal void removed ();
        public signal void bookmark (bool is_bookmarked);

        public Notebook             notebook;

        public Gtk.TextBuffer        title;
        public Gtk.TextBuffer        content;
        public bool                 bookmarked;

        DateTime _datetime;
        public DateTime             datetime {
            get {
                return _datetime;
            }
            set {
                _datetime = value;
                month = value.get_month ();
                year  = value.get_year ();
            }
        }

        public int          id;
        public int          month;
        public int          year;

        public bool is_brand_new;
        public bool is_trashed;
        public bool is_changed;

        public Gdk.Atom format;
        public Gdk.Atom deformat;

        public Note (Notebook notebook) {
            this.notebook = notebook;
            datetime = new DateTime.now_local ();
            build ();

            title.text = _("New Note");

            Footnote.DataBase.DatabaseManager.get_default ().add_note (this);

            events ();
        }

        public Note.from_existing (Notebook notebook,
            string title, uint8[] content_, int id, DateTime datetime, bool bookmarked) {
            this.bookmarked = bookmarked;
            this.notebook = notebook;
            this.id = id;
            this.datetime = datetime;
            build ();

            this.title.text = title;

            if (content_ != null) {
                Gtk.TextIter s;
                content.get_start_iter (out s);
                try {
                    content.deserialize (content, deformat, s, content_);
                } catch (Error e) {
                    critical (e.message);
                }
            }

            events ();
        }

        public void tag (string name, bool add) {
            Gtk.TextIter s, e;
            content.get_selection_bounds (out s, out e);
            if (add)
                content.apply_tag_by_name (name, s, e);
            else
                content.remove_tag_by_name (name, s, e);
        }

        public void clear_marked_color () {
            tag ("marked_green", false);
            tag ("marked_yellow", false);
            tag ("marked_red", false);
            tag ("marked_blue", false);
            tag ("marked_orange", false);
            tag ("marked_purple", false);
        }

        private void build () {
            content = new Gtk.TextBuffer (null);
            content.create_tag ("bold", "weight", Pango.Weight.BOLD);
            content.create_tag ("italic", "style", Pango.Style.ITALIC);
            content.create_tag ("strikethrough", "strikethrough", true);
            content.create_tag ("underline", "underline", Pango.Underline.SINGLE);
            content.create_tag ("marked_yellow", "background", "#fbd25d");
            content.create_tag ("marked_red", "background", "#da4d45");
            content.create_tag ("marked_green", "background", "#b3cf3b");
            content.create_tag ("marked_blue", "background", "#47a8e9");
            content.create_tag ("marked_orange", "background", "#f37329");
            content.create_tag ("marked_purple", "background", "#c35cc2");

            format = content.register_serialize_tagset ("default");
            deformat = content.register_deserialize_tagset ("default");

            is_brand_new = true;
            is_changed = false;

            title = new Gtk.TextBuffer (null);
        }

        private void events () {
            content.changed.connect ( () => { this.changed (); });
            title.changed.connect ( () => { this.changed (); });
            bookmark.connect (() => { this.changed (); });

            this.changed.connect ( () => { is_changed = true; });
            this.saved.connect ( () => { is_changed = false; });
            this.restored.connect ( () => { is_changed = false; });
            this.bookmark.connect ( () => { this.save (); });
        }

        public string save_temp_file () {
            var path = GLib.Environment.get_tmp_dir () + "/" + this.title.text + ".txt";
            debug ("Saving temp file to %s", path);
            try {
                GLib.File file = GLib.File.new_for_path (path);
                uint8[] data = content.text.data;
                string s;
                file.replace_contents (data, null, false, 0, out s);
            } catch (GLib.Error e) { warning (e.message); }
            return path;
        }

        public string get_time_label () {

            // If AM/PM doesn't exist, use 24h.
            string time_format;
            if (Posix.nl_langinfo (Posix.NLItem.AM_STR) == null || Posix.nl_langinfo (Posix.NLItem.AM_STR) == "") {
                time_format = Granite.DateTime.get_default_time_format (false);
            } else {

                // If AM/PM exists, assume it is the default time format and check for format override.
                var setting = new GLib.Settings ("org.gnome.desktop.interface");
                var clockformat = setting.get_user_value ("clock-format");

                if (clockformat == null || clockformat.get_string ().contains ("12h"))
                    time_format = Granite.DateTime.get_default_time_format (true);
                else
                    time_format = Granite.DateTime.get_default_time_format (false);
            }

            var date_format = Granite.DateTime.get_default_date_format (false, true, true);
            return datetime.format (date_format + " " + time_format);
        }

        public void save () {
            Footnote.DataBase.DatabaseManager.get_default ().update_note (this);
            if (this.is_brand_new == true)
                this.is_brand_new = false;
        }

        public void restore () {
            Footnote.DataBase.DatabaseManager.get_default ().restore_note (this);
        }

        public void remove () {
            Footnote.DataBase.DatabaseManager.get_default ().delete_note (this);
            this.removed ();
        }
    }
}
