//
//  Copyright (C) 2011-2012 Tom Beckmann
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

namespace Footnote.Objects {
    /**
     * Class represeting a Notebook with its notes
     **/
    public class Notebook : GLib.Object {
        public signal void note_added (Note note);
        public signal void note_changed (Note note);
        public signal void note_saved (Note note);
        public signal void note_trashed (Note note);
        public signal void note_removed (Note note);
        public signal void note_bookmark (Note note, bool is_bookmarked);
        public signal void modified ();
        public Footnote.Widgets.NoteView view;
        public int id;
        public string title;
        public int count;
        public Footnote.Widgets.NotebookItem sidebar_item;
        private Gdk.RGBA _color = Gdk.RGBA ();
        public Gdk.RGBA color {
            set {
                _color = value;
                icon = get_color_dot (color.red, color.green, color.blue, 16, 16);
                Footnote.DataBase.DatabaseManager.get_default ().update_notebook (this);
            } public get {
                return _color;
            }
        }
        public GLib.Icon icon;
        private Gee.ArrayList<Note> notes;
        private FootnoteApp app;

        construct {
            notes = new Gee.ArrayList<Note> ();
        }

        public Notebook.from_existing (FootnoteApp app, string title, Gdk.RGBA color, int id) {
            this.app = app;
            this.id = id;
            this.title = title;
            this.color = color;
            notes = new Gee.ArrayList<Note> ();

            this.build (app);
        }
        
        public Notebook (FootnoteApp app, string title){
            if (title == "") //dont want unnamed notebook
                this.title = _("Untitled");
            this.app = app;
            this.title = title;
            Footnote.DataBase.DatabaseManager.get_default ().add_notebook (this);

            this.build (app);

            view.show_all ();
            view.update_view ();
        }

        private void build (FootnoteApp app) {
            view = new Footnote.Widgets.NoteView (app, this);

            var _notes = DataBase.DatabaseManager.get_default ().get_notes_from_notebook (this);
            foreach (var note in _notes) {
                add_note (note);
            }
        }

        public void add_note (Note n) {
            if (notes.size > 0)
                notes.insert (1, n);
            else
                notes.insert (0, n);

            view.add_note (n);
            note_added (n);
            n.changed.connect (() => { note_changed (n);});
            n.saved.connect (() => { note_saved (n);});
            n.trashed.connect (() => { note_trashed (n);});
            n.removed.connect (() => {
                note_removed (n);
                delete_note (n);
            });

            n.bookmark.connect ((b) => { note_bookmark (n, b);});
        }

        public void delete_note (Note n) {
            notes.remove (n);
            view.update_view ();
        }

        public Gee.ArrayList<Note> get_notes () {
            var arraylist = new Gee.ArrayList<Note> ();
            arraylist.add_all (notes.read_only_view);
            return arraylist;
        }

        public bool is_empty () {
            if (notes.size == 0)
                return true;

            if (notes.size == 1)
                return notes.first ().is_trashed;

            return false;
        }

        public static Gdk.Pixbuf get_color_dot (double r, double g, double b,int w, int h) {
            var surface = new Cairo.ImageSurface (Cairo.Format.ARGB32, w, h);
            var ctx     = new Cairo.Context (surface);
            
            var grad = new Cairo.Pattern.linear (0, 0, 0, h);
            grad.add_color_stop_rgb (0, r, g, b);
            grad.add_color_stop_rgb (1, r*0.8, g*0.8, b*0.8);
            
            var grad2 = new Cairo.Pattern.radial (w/2, 0, 1, w/2, 0, w);
            grad2.add_color_stop_rgba (0, 1, 1, 1, 0.6);
            grad2.add_color_stop_rgba (1, 1, 1, 1, 0.0);
            
            ctx.set_line_width (1);
            
            Granite.Drawing.Utilities.cairo_rounded_rectangle (ctx, 0.5, 0.5, w-1, h-1, 3);
            ctx.set_source (grad);
            ctx.fill_preserve ();
            ctx.set_source (grad2);
            ctx.fill_preserve ();
            ctx.set_source_rgba (0, 0, 0, 0.3);
            ctx.stroke ();
            
            return Gdk.pixbuf_get_from_surface (surface, 0, 0, w, h);
        }
    }
}
