//
//  Copyright (C) 2011-2012 Andrea Basso
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

namespace Footnote.Widgets {
    /**
     * class representing an export dialog for both notes and notebooks
     **/
    public class ExportDialog : Gtk.FileChooserDialog {
        AdvancedOptions grid;
        bool[] prefs;
        GLib.HashTable<string, string> files;
        string current_ext = "";

        const string DEFAULT = "application/pdf";
        const int PREFS_NUM = 4;

        signal void filter_changed ();

        public ExportDialog.export_notes (Gee.ArrayList<Footnote.Objects.Note> _notes, string title, Gtk.Window window) {
            var notes = _notes;
            string[] bins = {"pdf"};

            files = new GLib.HashTable<string, string> (str_hash, str_equal);
            files["text/plain"] = "Plain Text (.txt)";
            files["text/rtf"] = "Rich Text Format (.rtf)";
            files["application/pdf"] = "Portable Document Format (.pdf)";
            build ();

            var default_name = files[DEFAULT];
            var default_ext = default_name[default_name.index_of ("(")+1:-1];
            set_current_name (title + default_ext);
            current_ext = default_ext;

            string header = "";
            string footer = "";
            string str = "";

            notify.connect ( (p)=> {
                if (p.get_name () == "filter")
                    filter_changed ();
            });

            response.connect ( (id)=> {
                if (id == Gtk.ResponseType.ACCEPT) {
                    string ext = current_ext[1:current_ext.length];
                    prefs = grid.get_prefs ();

                    // We need a reversed array…
                    if (!prefs[3]) {
                        var reversed_notes = new Gee.ArrayList<Footnote.Objects.Note> ();
                        for (int i = notes.size; i>0; i--) {
                            reversed_notes.add (notes.get (i-1));
                        }

                        notes = reversed_notes;
                    }

                    if (ext in bins) {
                        switch (ext) {
                            case "pdf":
                                var op = new CustomPrint (notes, prefs);
                                op.set_export_filename (this.get_filename ());
                                try {
                                    op.run (Gtk.PrintOperationAction.EXPORT, window);
                                } catch (Error e) { warning (e.message); }
                                break;
                        }
                    } else {
                        try {
                            var output = File.new_for_path (this.get_filename ());
                            try {output.delete ();} catch (GLib.Error e) {};
                            var data_stream = new DataOutputStream (output.create (FileCreateFlags.NONE));
                            switch (ext) {
                                case "rtf":
                                    header = "{\\rtf1\\ansi{\\fonttbl\\f0\\fswiss Open Sans;}\\f0\n{\\colortbl;\\red0\\green0\\blue0;\\red127\\green127\\blue127;}\n";
                                    footer = "}";
                                    str = header;
                                    foreach (Footnote.Objects.Note n in notes) {
                                        str += rtf (n);
                                    }
                                    str += footer;
                                    break;
                                case "txt":
                                    foreach (Footnote.Objects.Note n in notes) {
                                        str += txt (n);
                                    }
                                    str += footer;
                                    break;
                            }
                            data_stream.put_string (str);
                        } catch (GLib.Error e) {
                            stderr.printf ("Error: %s\n", e.message);
                        }
                    }
                    this.destroy ();
                }
            });
        }

        private void build () {
            set_action (Gtk.FileChooserAction.SAVE);
            set_title ("");
            set_create_folders (true);
            set_do_overwrite_confirmation (true);
            set_default_response (Gtk.ResponseType.ACCEPT);

            var entry = new Gtk.Entry ();
            ((Gtk.Box)get_child ()).get_children ().foreach ( (w)=> {       // it's an awful solution, but w/out it
                ((Gtk.Container)w).foreach ( (d)=> {                        // if the user changed file type during "last used"
                    if (d as Gtk.Container != null) {                       // or "search" we couldn't update the extension
                      ((Gtk.Container)d).foreach ( (f)=> {
                            if (f as Gtk.Container != null) {
                                ((Gtk.Container)f).foreach ( (g)=> {
                                    if (g as Gtk.Container != null) {
                                        ((Gtk.Container)g).foreach ( (h)=> {
                                            if (h as Gtk.Entry != null) {
                                                entry = (Gtk.Entry)h;
                                            }
                                        });
                                    }
                                });
                            }
                        });
                    }
                });
            });

            filter_changed.connect ( ()=> {
                var name = entry.get_text ();
                if (name != null) {
                    var ext_name = this.get_filter ().get_filter_name ();
                    var ext = ext_name[ext_name.index_of ("(") + 1 : -1];
                    if (name.has_suffix (current_ext))
                        set_current_name (name[0:-4] + ext);
                    else
                        set_current_name (name + ext);
                    current_ext = ext;
                }
            });

            var cancel = (Gtk.Button)add_button (_("Cancel"), 1);
            cancel.clicked.connect ( ()=> { this.destroy (); });
            add_button (_("Save"), Gtk.ResponseType.ACCEPT);

            files.foreach ( (mime, name) => {
                var filter = new Gtk.FileFilter ();
                filter.add_mime_type (mime);
                filter.set_filter_name (name);
                this.add_filter (filter);
                if (DEFAULT in mime)
                    this.set_filter (filter);
            });

            var adv = new Gtk.Expander (_("Advanced Options"));
            grid = new AdvancedOptions (PREFS_NUM);
            adv.add (grid);
            this.set_extra_widget (adv);

            this.show_all ();
        }

        private string txt (Footnote.Objects.Note n) {
            string r = "";
            if (prefs[0])
                r += n.title.text + "\n";
            if (prefs[2])
                r += n.get_time_label () + "\n\n";
            if (prefs[1])
                r += n.content.text + "\n\n\n";
            return r;
        }

        private string rtf (Footnote.Objects.Note n) {
            string r = "";
            if (prefs[0]) {
                string title = n.title.text;
                string note_title = @"\\cf1\n\\pard{\\b $title}\\par\n";
                r += note_title;
            }
            if (prefs[2]) {
                string date = n.get_time_label ();
                string note_date = @"\\cf2\n\\pard{$date}\\par\\line\n";
                r += note_date;
            }
            if (prefs[1]) {
                string body = n.content.text.replace ("\\", "\\\\");
                body = body.replace("\n", "\\line\n");
                string note_body = @"\\cf1\n\\pard $body\\par\n\\line\\line";
                r += note_body;
            }
            return r;
        }
    }
}
