//
//  Copyright (C) 2011-2012 Tom Beckmann
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//  Authored by:    Artem Anufrij <artem.anufrij@live.de>

namespace Footnote.Widgets {

    public class ActionMenu : Gtk.Revealer {

        public Gtk.Button delete_button;
        public Gtk.Button bookmarked_button;

        public Footnote.Objects.Note note { get; set; }

        public ActionMenu (Footnote.Objects.Note? note, Gtk.Orientation orientation) {

            delete_button = new Gtk.Button.from_icon_name ("edit-delete-symbolic", Gtk.IconSize.BUTTON);
            delete_button.tooltip_text = _("Delete");
            delete_button.relief = Gtk.ReliefStyle.NONE;

            bookmarked_button = new Gtk.Button.from_icon_name ("non-starred", Gtk.IconSize.BUTTON);
            bookmarked_button.relief = Gtk.ReliefStyle.NONE;

            this.notify["note"].connect ( () => {
                if (note != null && note.bookmarked) {
                    bookmarked_button.image = new Gtk.Image.from_icon_name ("starred", Gtk.IconSize.BUTTON);
                } else {
                    bookmarked_button.image = new Gtk.Image.from_icon_name ("non-starred", Gtk.IconSize.BUTTON);
                }
            });

            if (note != null)
                this.note = note;

            var buttons = new Gtk.Grid ();
            buttons.orientation = orientation;
            buttons.add (bookmarked_button);
            buttons.add (delete_button);

            buttons.opacity = 0.5;
            this.transition_type = Gtk.RevealerTransitionType.CROSSFADE;
            this.add (buttons);

            /**
             * Events
             **/

            /*delete note*/
            delete_button.clicked.connect ( () => {
                this.note.is_trashed = true;
                this.note.trashed ();
            });

            /*bookmark note*/
            bookmarked_button.clicked.connect (() => {
                this.note.bookmarked = !this.note.bookmarked;
                this.note.bookmark (this.note.bookmarked);
                if (this.note.bookmarked) {
                    bookmarked_button.image = new Gtk.Image.from_icon_name ("starred", Gtk.IconSize.BUTTON);
                } else {
                    bookmarked_button.image = new Gtk.Image.from_icon_name ("non-starred", Gtk.IconSize.BUTTON);
                }
                bookmarked_button.show_all ();
            });
        }
    }

    /**
     * Display a (filtered) list of Notes for the currently
     * selected Category.
     */
    public class NoteView : Gtk.Box {
        public signal void start_editing ();
        public signal void stop_editing ();
        public string filter;

        private FootnoteApp app;

        private Gtk.Paned           miller_paned;
        private Gtk.ListBox         miller_listbox;
        private Gtk.ScrolledWindow  miller_scroll;
        private Gtk.ScrolledWindow  miller_view_scroll;
        public EditView             note_editor;

        private Footnote.Objects.Notebook            notebook;
        private Gtk.Stack           main_stack;

        private Gtk.InfoBar         deletion_infobar;
        private Gtk.Label           deletion_label;
        private Footnote.Objects.Note                trashed_note;

        private int year = -1;
        private int month = -1;

        /**
         * Constructor
         */
        public NoteView (FootnoteApp app, Footnote.Objects.Notebook notebook) {
            this.orientation    = Gtk.Orientation.VERTICAL;
            this.app            = app;
            this.notebook       = notebook;
            this.filter         = "";

            deletion_infobar = new Gtk.InfoBar ();
            deletion_infobar.message_type = Gtk.MessageType.INFO;
            deletion_infobar.no_show_all = true;
            deletion_infobar.add_button (_("Undo"), Gtk.ResponseType.CANCEL);

            var close_button = deletion_infobar.add_button (_("Dismiss"), Gtk.ResponseType.CLOSE);
            close_button.get_style_context ().add_class (Gtk.STYLE_CLASS_DESTRUCTIVE_ACTION);

            deletion_label = new Gtk.Label (null);
            deletion_infobar.get_content_area ().add (deletion_label);
            deletion_infobar.response.connect ((id) => {
                switch (id) {
                    case Gtk.ResponseType.CANCEL:
                        trashed_note.is_trashed = false;
                        update_notes_visibility.begin ();
                        update_view ();
                        trashed_note = null;
                        break;
                    case Gtk.ResponseType.CLOSE:
                        delete_trashed_note ();
                        break;
                }

                deletion_infobar.no_show_all = true;
                deletion_infobar.hide ();
            });

            miller_paned = new Gtk.Paned (Gtk.Orientation.HORIZONTAL);
            miller_paned.position = Settings.get_default ().miller_handle_position;

            miller_listbox = new Gtk.ListBox ();
            note_editor = new EditView (app);

            miller_scroll = new Gtk.ScrolledWindow (null, null);
            miller_scroll.add (note_editor);

            miller_listbox.row_activated.connect ((row) => {
                open_note (((FootnoteRow)row).note);
            });

            miller_listbox.set_filter_func ((row) => {
                return should_appear (((FootnoteRow)row).note);
            });

            var empty_label = new Gtk.Label (_("Notebook \"%s\" is empty").printf (notebook.title));
            empty_label.expand = true;
            empty_label.halign = Gtk.Align.CENTER;
            empty_label.valign = Gtk.Align.CENTER;
            var empty_grid = new Gtk.Grid ();
            empty_grid.get_style_context ().add_class (Gtk.STYLE_CLASS_VIEW);
            empty_grid.expand = true;
            empty_grid.add (empty_label);

            miller_view_scroll = new Gtk.ScrolledWindow (null, null);
            miller_view_scroll.add (miller_listbox);
            miller_view_scroll.hscrollbar_policy = Gtk.PolicyType.NEVER;

            miller_paned.pack1 (miller_view_scroll, false, false);
            miller_paned.pack2 (miller_scroll, true, false);

            main_stack = new Gtk.Stack ();
            main_stack.transition_type = Gtk.StackTransitionType.SLIDE_LEFT_RIGHT;
            main_stack.add_named (empty_grid, "empty");
            main_stack.add_named (miller_paned, "miller");

            add (deletion_infobar);
            add (main_stack);

            show_all ();
        }

        /*
         * only for list view
         **/
        public void toggle_edit (bool show) {
            if (show) {
                start_editing ();
            } else {
                stop_editing ();
            }
        }

        /**
         * Add a note to the view
         */
        public void add_note (Footnote.Objects.Note n, int pos=0) {
            update_view ();

            var row = new FootnoteRow (n);
            miller_listbox.add (row);

            n.trashed.connect (() => {
                trash_note (n);
            });
            show_all ();

            row.get_style_context ().remove_class("button");
        }

        /**
         * Remove a note from the view
         */
        public void trash_note (Footnote.Objects.Note n) {
            deletion_label.label = _("Note \"%s\" removed.").printf (n.title.text);
            deletion_infobar.no_show_all = false;
            deletion_infobar.show_all ();
            delete_trashed_note ();
            trashed_note = n;
            update_notes_visibility.begin ();
            if (notebook.is_empty ()) {
                start_editing ();
                main_stack.set_visible_child_name ("empty");
            }
        }

        /**
         * Update the view, loop through ArrayList of Notes and
         * if filter isn't empty, only show notes containing the
         * filter string (e.g. in title, text or as a tag).
         */
        public void update_view () {
            if (notebook.is_empty ()) {
                start_editing ();
                main_stack.set_visible_child_name ("empty");
                return;
            }

            if (miller_listbox.get_selected_row () == null) {
                var row = miller_listbox.get_row_at_index (0);
                if (row != null) {
                    miller_listbox.select_row (row);
                    miller_listbox.row_activated (row);
                }
            } else
                 miller_view_scroll.show ();

            main_stack.set_visible_child_name ("miller");
        }

        public void change_month (int month) {
            this.month = month;
            update_notes_visibility.begin ();
        }

        public void change_year (int year) {
            this.year = year;
            update_notes_visibility.begin ();
        }

        private void delete_trashed_note () {
            if (trashed_note == null)
                return;

            trashed_note.remove ();
            trashed_note = null;
        }

        private async void update_notes_visibility () {
            miller_listbox.invalidate_filter ();
        }

        private bool should_appear (Footnote.Objects.Note? note) {
            if (note == null || note.is_trashed)
                return false;

            if ((
                (month == -1 || month == note.month) &&
                (year  == -1 || year  == note.year )) &&
                (filter == "" ||
                 filter.down () in note.title.text.down () ||
                 filter.down () in note.content.text.down ())) {
                     return true;
                 }
            return false;
        }

        /**
         * Scroll to a certain note in the list.
         */
        public void open_note (Footnote.Objects.Note n, bool bookmarked = false) {
            toggle_edit (true);

            note_editor.open_note (n);

            if (bookmarked)
                miller_view_scroll.hide ();
            else
                miller_view_scroll.show ();
        }

        public void select_note (Footnote.Objects.Note n) {
            miller_listbox.@foreach ( (row) => {
                if ((row as FootnoteRow).note == n) {
                    row.activate ();
                    return;
                }
            });
        }
    }

    public class FootnoteRow : Gtk.ListBoxRow {
        public Footnote.Objects.Note note;
        private Gtk.Label title_label;
        private Gtk.Label date_label;
        private Gtk.Label summary_label;

        public FootnoteRow (Footnote.Objects.Note note) {
            this.note = note;
            title_label = new Gtk.Label ("");
            title_label.get_style_context ().add_class ("h3");
            title_label.halign = Gtk.Align.START;
            title_label.ellipsize = Pango.EllipsizeMode.END;

            date_label = new Gtk.Label ("");
            date_label.opacity = 0.8;
            date_label.ellipsize = Pango.EllipsizeMode.END;
            date_label.use_markup = true;
            date_label.halign = Gtk.Align.END;
            date_label.margin_right = 6;

            summary_label = new Gtk.Label ("");
            summary_label.halign = Gtk.Align.START;
            summary_label.valign = Gtk.Align.START;
            summary_label.ellipsize = Pango.EllipsizeMode.END;
            summary_label.use_markup = true;
            summary_label.opacity = 0.8;
            summary_label.margin_right = 6;
            summary_label.expand = true;

            var buttons_revealer = new ActionMenu (note, Gtk.Orientation.VERTICAL);
            buttons_revealer.valign = Gtk.Align.START;
            var grid = new Gtk.Grid ();
            grid.margin_top = 4;
            grid.row_spacing = 4;
            grid.attach (buttons_revealer, 0, 0, 1, 2);
            grid.attach (title_label, 1, 0, 1, 1);
            grid.attach (date_label, 2, 0, 1, 1);
            grid.attach (summary_label, 1, 1, 2, 1);
            grid.attach (new Gtk.Separator (Gtk.Orientation.HORIZONTAL), 0, 3, 3, 1);

            var event_box = new Gtk.EventBox ();
            event_box.add (grid);
            event_box.events |= Gdk.EventMask.ENTER_NOTIFY_MASK|Gdk.EventMask.LEAVE_NOTIFY_MASK;

            event_box.enter_notify_event.connect ((event) => {
                buttons_revealer.set_reveal_child (true);
                return false;
            });

            event_box.leave_notify_event.connect ((event) => {
                if (event.detail == Gdk.NotifyType.INFERIOR)
                    return false;
                buttons_revealer.set_reveal_child (false);
                return false;
            });

            add (event_box);
            this.show_all();
            update_labels ();

            note.saved.connect (() => {
                update_labels ();
            });
        }

        public void update_labels () {

            string [] lines = note.content.text.split ("\n");

            var result = "";
            var result_lines = 0;
            foreach (string line in lines) {
                if (line.strip ().length == 0)
                    continue;

                result += line + "\n";
                result_lines ++;

                if (result_lines == 3)
                    break;
            }

            summary_label.label =  "<span font_size=\"small\">" + result.strip () + "</span>";;
            date_label.label = "<span font_size=\"small\">" + note.get_time_label () + "</span>";
            title_label.label = note.title.text;
        }
    }
}
